# Generated by roxygen2: do not edit by hand

S3method(plot,dsp)
S3method(predict,dsp)
S3method(print,dsp)
S3method(print,dsp_spec)
S3method(summary,dsp)
export(dsp_fit)
export(dsp_spec)
export(simRegression)
export(simUnivariate)
import(Matrix)
import(coda)
import(pgdraw)
import(rlang)
import(stochvol)
importFrom(BayesLogit,rpg)
importFrom(MCMCpack,rinvgamma)
importFrom(Matrix,bandSparse)
importFrom(Matrix,chol)
importFrom(Matrix,solve)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(RcppZiggurat,zrnorm)
importFrom(coda,as.mcmc)
importFrom(coda,effectiveSize)
importFrom(fda,create.bspline.basis)
importFrom(fda,eval.basis)
importFrom(glue,glue)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(lifecycle,deprecated)
importFrom(methods,as)
importFrom(methods,is)
importFrom(mgcv,rig)
importFrom(msm,rtnorm)
importFrom(pgdraw,pgdraw)
importFrom(progress,progress_bar)
importFrom(purrr,map)
importFrom(spam,as.spam.dgCMatrix)
importFrom(spam,chol)
importFrom(spam,rmvnorm.canonical)
importFrom(stats,approxfun)
importFrom(stats,arima)
importFrom(stats,arima.sim)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,dgamma)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,mad)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stochvol,svsample_fast_cpp)
importFrom(truncdist,rtrunc)
useDynLib(dsp)
