# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

draw_indicators_generic <- function(res, r, n, w, m, s, J) {
    .Call('_dsp_draw_indicators_generic', PACKAGE = 'dsp', res, r, n, w, m, s, J)
}

sample_mat <- function(row_ind, col_ind, val, n, num_entries, linht, rd, D) {
    .Call('_dsp_sample_mat', PACKAGE = 'dsp', row_ind, col_ind, val, n, num_entries, linht, rd, D)
}

